@@ -10,12 +10,12 @@ AthenaResult <- function(conn,
   response <- list(QueryExecutionId = NULL)
   if (athena_option_env$cache_size > 0) response <- list(QueryExecutionId = check_cache(statement, conn@info$work_group))
   if (is.null(response$QueryExecutionId)) {
-  tryCatch(response <- conn@ptr$Athena$start_query_execution(QueryString = statement,
+  retry_api_call(response <- conn@ptr$Athena$start_query_execution(QueryString = statement,
                                                              QueryExecutionContext = list(Database = conn@info$dbms.name),
                                                              ResultConfiguration = ResultConfiguration(conn),
                                                              WorkGroup = conn@info$work_group))}
   on.exit(if(!is.null(conn@info$expiration)) time_check(conn@info$expiration))
-  new("AthenaResult", connection = conn, info = response)
+  new("AthenaResult", connection = conn, info = c(response, list(NextToken = NULL)))
 }
 
 #' @rdname AthenaConnection
@@ -38,7 +38,7 @@ setClass(
 #' @return \code{dbClearResult()} returns \code{TRUE}, invisibly.
 #' @seealso \code{\link[DBI]{dbIsValid}}
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' # Note: 
 #' # - Require AWS Account to run below example.
 #' # - Different connection methods can be used please see `noctua::dbConnect` documnentation
@@ -67,11 +67,11 @@ setMethod(
     } else {
       
       # checks status of query
-      tryCatch(query_execution <- res@connection@ptr$Athena$get_query_execution(QueryExecutionId = res@info$QueryExecutionId))
+      retry_api_call(query_execution <- res@connection@ptr$Athena$get_query_execution(QueryExecutionId = res@info$QueryExecutionId))
       
       # stops resource if query is still running
       if (!(query_execution$QueryExecution$Status$State %in% c("SUCCEEDED", "FAILED", "CANCELLED"))){
-        tryCatch(res@connection@ptr$Athena$stop_query_execution(QueryExecutionId = res@info$QueryExecutionId))}
+        retry_api_call(res@connection@ptr$Athena$stop_query_execution(QueryExecutionId = res@info$QueryExecutionId))}
       
       # clear s3 athena output
       # split s3_uri
@@ -88,12 +88,16 @@ setMethod(
                  error = function(e) warning(e, call. = F))
         tryCatch(res@connection@ptr$S3$delete_object(Bucket = result_info$bucket,
                                                      Key = result_info$key),
-                 error = function(e) cat(""))}
+                 error = function(e) cat(""))
+        # remove manifest csv created with CTAS statements 
+        if (query_execution$QueryExecution$StatementType == "DDL")
+          tryCatch(res@connection@ptr$S3$delete_object(Bucket = result_info$bucket,
+                                                       Key = paste0(result_info$key, "-manifest.csv")),
+                   error = function(e) cat(""))}
     }
     invisible(TRUE)
   })
 
-
 #' Fetch records from previously executed query
 #' 
 #' Currently returns the top n elements (rows) from result set or returns entire table from Athena.
@@ -105,7 +109,7 @@ setMethod(
 #' @return \code{dbFetch()} returns a data frame.
 #' @seealso \code{\link[DBI]{dbFetch}}
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' # Note: 
 #' # - Require AWS Account to run below example.
 #' # - Different connection methods can be used please see `noctua::dbConnect` documnentation
@@ -144,16 +148,58 @@ setMethod(
       stop(result$QueryExecution$Status$StateChangeReason, call. = FALSE)
     }
     
+    # return metadata of athena data types
+    retry_api_call(result_class <- res@connection@ptr$Athena$get_query_results(QueryExecutionId = res@info$QueryExecutionId,
+                                                                               MaxResults = as.integer(1))$ResultSet$ResultSetMetadata$ColumnInfo)
     if(n >= 0 && n !=Inf){
-      n = as.integer(n + 1)
-      if (n > 1000){n = 1000L; message("Info: n has been restricted to 1000 due to AWS Athena limitation")}
-      tryCatch(result <- res@connection@ptr$Athena$get_query_results(QueryExecutionId = res@info$QueryExecutionId, MaxResults = n))
+      # assign token from AthenaResult class
+      token <- res@info$NextToken
       
-      output <- lapply(result$ResultSet$Rows, function(x) (sapply(x$Data, function(x) if(length(x) == 0 ) NA else x)))
-      dt <- rbindlist(output, fill = TRUE)
-      colnames(dt) <- as.character(unname(dt[1,]))
-      rownames(dt) <- NULL
-      return(dt[-1,])
+      if(length(token) == 0) n <- as.integer(n + 1)
+      chunk <- as.integer(n)
+      if (n > 1000L) chunk = 1000L
+      
+      iterate <- 1:ceiling(n/chunk)
+      
+      # create empty list shell
+      dt_list <- list()
+      length(dt_list) <- max(iterate)
+      
+      for (i in iterate){
+        if(i == max(iterate)) chunk <- as.integer(n - (i-1) * chunk)
+        
+        # get chunk with retry api call if call fails
+        retry_api_call(result <- res@connection@ptr$Athena$get_query_results(QueryExecutionId = res@info$QueryExecutionId, NextToken = token, MaxResults = chunk))
+        
+        # process returned list
+        output <- lapply(result$ResultSet$Rows, function(x) (sapply(x$Data, function(x) if(length(x) == 0 ) NA else x)))
+        suppressWarnings(staging_dt <- rbindlist(output, use.names = FALSE))
+        
+        # remove colnames from first row
+        if (i == 1 && length(token) == 0) {
+          staging_dt <- staging_dt[-1,]
+        }
+        
+        # ensure rownames are not set
+        rownames(staging_dt) <- NULL
+        
+        # added staging data.table to list
+        dt_list[[i]] <- staging_dt
+        
+        # if token hasn't changed or if no more tokens are available then break loop
+        if ((length(token) != 0 && token == result$NextToken) || length(result$NextToken) == 0) {break} else {token <- result$NextToken}
+      }
+      
+      # combined all lists together
+      dt <- rbindlist(dt_list, use.names = FALSE)
+      
+      # Update last token in s4 class
+      eval.parent(substitute(res@info$NextToken <- result$NextToken))
+      
+      # replace names with actual names
+      Names <- sapply(result_class, function(x) x$Name)
+      colnames(dt) <- Names
+      return(dt)
     }
     
     # Added data scan information when returning data from athena
@@ -165,14 +211,10 @@ setMethod(
     
     # connect to s3 and create a bucket object
     # download athena output
-    tryCatch(obj <- res@connection@ptr$S3$get_object(Bucket = result_info$bucket, Key = result_info$key))
+    retry_api_call(obj <- res@connection@ptr$S3$get_object(Bucket = result_info$bucket, Key = result_info$key))
     
     write_bin(obj$Body, File)
     
-    # return metadata of athena data types
-    tryCatch(result_class <- res@connection@ptr$Athena$get_query_results(QueryExecutionId = res@info$QueryExecutionId,
-                                                                         MaxResults = as.integer(1))$ResultSet$ResultSetMetadata$ColumnInfo)
-
     if(grepl("\\.csv$",result_info$key)){
       output <- athena_read(athena_option_env$file_parser, File, result_class)
     } else{
@@ -196,7 +238,7 @@ setMethod(
 #' @return \code{dbHasCompleted()} returns a logical scalar. \code{TRUE} if the query has completed, \code{FALSE} otherwise.
 #' @seealso \code{\link[DBI]{dbHasCompleted}}
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' # Note: 
 #' # - Require AWS Account to run below example.
 #' # - Different connection methods can be used please see `noctua::dbConnect` documnentation
@@ -224,7 +266,7 @@ setMethod(
   "dbHasCompleted", "AthenaResult",
   function(res, ...) {
     if (!dbIsValid(res)) {stop("Result already cleared", call. = FALSE)}
-    tryCatch(query_execution <- res@connection@ptr$Athena$get_query_execution(QueryExecutionId = res@info$QueryExecutionId))
+    retry_api_call(query_execution <- res@connection@ptr$Athena$get_query_execution(QueryExecutionId = res@info$QueryExecutionId))
     
     if(query_execution$QueryExecution$Status$State %in% c("SUCCEEDED", "FAILED", "CANCELLED")) TRUE
     else if (query_execution$QueryExecution$Status$State == "RUNNING") FALSE
@@ -259,7 +301,7 @@ setMethod(
 #'         The data.frame has two columns (field_name, type).
 #' @seealso \code{\link[DBI]{dbHasCompleted}}
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' # Note: 
 #' # - Require AWS Account to run below example.
 #' # - Different connection methods can be used please see `RAthena::dbConnect` documnentation
@@ -291,7 +333,7 @@ setMethod(
       stop(result$QueryExecution$Status$StateChangeReason, call. = FALSE)
     }
     
-    tryCatch(result <- res@connection@ptr$Athena$get_query_results(QueryExecutionId = res@info$QueryExecutionId,
+    retry_api_call(result <- res@connection@ptr$Athena$get_query_results(QueryExecutionId = res@info$QueryExecutionId,
                                                                    MaxResults = as.integer(1)))
     
     Name <- sapply(result$ResultSet$ResultSetMetadata$ColumnInfo, function(x) x$Name)
@@ -308,7 +350,7 @@ setMethod(
 #' @name dbStatistics
 #' @return \code{dbStatistics()} returns list containing Athena Statistics return from \code{paws}.
 #' @examples 
-#' \donttest{
+#' \dontrun{
 #' # Note: 
 #' # - Require AWS Account to run below example.
 #' # - Different connection methods can be used please see `RAthena::dbConnect` documnentation
